from .pricing import get_az
from .spot_instance import request
from .instance import tag as tag_instance
from .instance import get_by_instance_id
from .client import build as build_client
from .config import Config
from .key import find_or_create as find_or_create_key


def launch(args):
    client = build_client(args)
    conf = Config(client, args)

    key_path = find_or_create_key(client, conf.key_name)

    az = get_az(client, conf)
    _log_launching(az)
    conf.set_az(az.zone_name)

    inst = request(client, conf, tag_instance, get_by_instance_id)
    _log_instance_creation(inst, key_path)
    return inst


def _log_launching(az):
    print(">> Launching instance in:")
    print(str(az))


def _log_instance_creation(instance, key_path):
    print(">> Instance launched, connect with:")
    ip = str(instance.ip_address)
    print("ssh -i " + key_path + " ubuntu@" + ip)



import os

def find_or_create(client, conf_key_name):
    path = os.path.expanduser('~/.ssh/' + conf_key_name + '.pem')
    if not os.path.exists(path):
        response = client.create_key_pair(conf_key_name)

        with os.fdopen(os.open(path, os.O_WRONLY | os.O_CREAT, 0o400), 'w') as handle:
            handle.write(response['KeyMaterial'])

    return path



from .snapshot import create_and_wait
from .instance import find_latest as find_latest_instance
from .instance import destroy as destroy_instance
from .client import build as build_client
from .config import Config


def destroy(args):
    client = build_client(args)
    conf = Config(client, args)
    instance = find_latest_instance(client, conf)
    _log_snapshot(instance)
    snap = create_and_wait(client, instance.volume_id)
    _log_destroying()
    return destroy_instance(client, instance.id)


def _log_snapshot(instance):
    print(">> Creating snapshot for instance:")
    print(str(instance.ip_address))


def _log_destroying():
    print(">> Destroying instance.")
